
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A button.
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsButton extends Canvas implements Serializable, MouseListener
{
 /**
  * Label is shown horizontally and vertically centered.
  */
 public static final int lpCenter = 0;
 /**
  * Label is shown in the upper left corner of the control.
  */
 public static final int lpTopLeft = 1;
 /**
  * Label is shown in the upper right corner of the control.
  */
 public static final int lpTopRight = 2;
 /**
  * Label is shown in the lower right corner of the control.
  */
 public static final int lpBottomRight = 3;
 /**
  * Label is shown in the lower left corner of the control.
  */
 public static final int lpBottomLeft = 4;

 private Image backImage;
 private String label;
 private Color fontColor;
 private Color enabledColor;
 private Color disabledColor;
 private Color pressedColor;
 private int labelPosition = lpCenter;
 private int labelMarginX;
 private int labelMarginY;
 private int labelPosX;
 private int labelPosY;
 private boolean roundLeft;
 private boolean roundRight;
 private int radiusLeft;
 private int radiusRight;
 private boolean mouseIsPressed;
 private Vector listeners = new Vector();

 /**
  * Constructs a new instance.
  */
 public LcarsButton()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  roundLeft = true;
  roundRight = true;
  labelMarginX = 5;
  labelMarginY = 2;
  radiusLeft = getSize().height / 2;
  radiusRight = getSize().height / 2;
  label = "";
  enabledColor = Color.blue;
  disabledColor = Color.gray;
  mouseIsPressed = false;
  pressedColor = Color.cyan;
  fontColor = Color.black;
  addMouseListener(this);
  setSize(60, 25);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {}

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  Color foreColor;
  int width = getSize().width;
  int height = getSize().height;
  if (roundLeft == false) radiusLeft = 0;
  else radiusLeft = height / 2;
  if (roundRight == false) radiusRight = 0;
  else radiusRight = height / 2;
  if (isEnabled()) foreColor = getEnabledColor();
  else foreColor = getDisabledColor();
  if ((mouseIsPressed) && (isEnabled())) foreColor = getPressedColor();
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.clearRect(0, 0, width, height);
  backBuffer.setColor(foreColor);
  backBuffer.fillRoundRect(0, 0, width, height, height, height);
  if (roundLeft == false)
   backBuffer.fillRect(0, 0, height / 2 + 1, height);
  if (roundRight == false)
   backBuffer.fillRect(width - (height / 2 + 1), 0, width, height);
  backBuffer.setColor(fontColor);
  FontMetrics fm = backBuffer.getFontMetrics();
  switch (labelPosition)
  {
   case lpCenter :
    labelPosX = (width - fm.stringWidth(label)) /2;
    labelPosY = (height + fm.getAscent() - fm.getDescent()) / 2;
    break;
   case lpTopLeft :
    labelPosX = radiusLeft + labelMarginX;
    labelPosY = labelMarginY + fm.getAscent();
    break;
   case lpTopRight :
    labelPosX = width - radiusRight - labelMarginX - fm.stringWidth(label);
    labelPosY = labelMarginY + fm.getAscent();
    break;
   case lpBottomRight :
    labelPosX = width - radiusRight - labelMarginX - fm.stringWidth(label);
    labelPosY = height - labelMarginY - fm.getDescent();
    break;
   case lpBottomLeft :
    labelPosX = radiusLeft + labelMarginX;
    labelPosY = height - labelMarginY - fm.getDescent();
    break;
  }
  backBuffer.drawString(label, labelPosX, labelPosY);
  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Returns the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }

 /**
  * Returns the color used for drawing the label.
  */
 public Color getFontColor()
 {
  return fontColor;
 }

 /**
  * Sets the color used for drawing the label.
  */
 public void setFontColor(Color newFontColor)
 {
  if (!fontColor.equals(newFontColor))
  {
   fontColor = newFontColor;
   repaint();
  }
 }

 /**
  * Returns the label.
  */
 public String getLabel()
 {
  return label;
 }

 /**
  * Sets the label.
  */
 public void setLabel(String newLabel)
 {
  if (!label.equals(newLabel))
  {
   label = newLabel;
   repaint();
  }
 }

 /**
  * Returns the position of the label.
  */
 public int getLabelPosition()
 {
  return labelPosition;
 }

 /**
  * Sets the position of the label.<BR>
  * Possible values: lpCenter, lpTopLeft, lpTopRight, lpBottomRight, lpBottomLeft<BR>
  * If the new value is not in the range between 0 and 4 the property is set to
  * lpCenter.
  */
 public void setLabelPosition(int newLabelPosition)
 {
  if (labelPosition != newLabelPosition)
  {
   if ((newLabelPosition < 0) || (newLabelPosition > 4))
    labelPosition = lpCenter;
   else
    labelPosition = newLabelPosition;
   repaint();
  }
 }

 /**
  * Returns the horizontal margin between the label and the bound of the control.
  */
 public int getLabelMarginX()
 {
  return labelMarginX;
 }

 /**
  * Sets the horizontal margin between the label and the bound of the control
  * if the label position is not lpCenter.
  */
 public void setLabelMarginX(int newLabelMarginX)
 {
  if (labelMarginX != newLabelMarginX)
  {
   labelMarginX = newLabelMarginX;
   repaint();
  }
 }

 /**
  * Returns the vertical margin between the label and the bound of the control.
  */
 public int getLabelMarginY()
 {
  return labelMarginY;
 }

 /**
  * Sets the vertical margin between the label and the bound of the control
  * if the label position is not lpCenter.
  */
 public void setLabelMarginY(int newLabelMarginY)
 {
  if (labelMarginY != newLabelMarginY)
  {
   labelMarginY = newLabelMarginY;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the control is enabled.
  */
 public Color getEnabledColor()
 {
  return enabledColor;
 }

 /**
  * Sets the color which is used when the control is enabled.
  */
 public void setEnabledColor(Color newEnabledColor)
 {
  if (!enabledColor.equals(newEnabledColor))
  {
   enabledColor = newEnabledColor;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the button is pressed.
  */
 public Color getPressedColor()
 {
  return pressedColor;
 }

 /**
  * Sets the color which is used when the button is pressed.
  */
 public void setPressedColor(Color newPressedColor)
 {
  if (!pressedColor.equals(newPressedColor))
  {
   pressedColor = newPressedColor;
   repaint();
  }
 }

 /**
  * Returns the color which is used when the control is not enabled.
  */
 public Color getDisabledColor()
 {
  return disabledColor;
 }

 /**
  * Sets the color which is used when the control is not enabled.
  */
 public void setDisabledColor(Color newDisabledColor)
 {
  if (!disabledColor.equals(newDisabledColor))
  {
   disabledColor = newDisabledColor;
   repaint();
  }
 }

 /**
  * Returns whether the left side of the control is drawn rounded.
  */
 public boolean isRoundLeft()
 {
  return roundLeft;
 }

 /**
  * Sets whether the left side of the control is drawn rounded.
  */
 public void setRoundLeft(boolean newRoundLeft)
 {
  if (roundLeft != newRoundLeft)
  {
   roundLeft = newRoundLeft;
   repaint();
  }
 }

 /**
  * Returns whether the right side of the control is drawn rounded.
  */
 public boolean isRoundRight()
 {
  return roundRight;
 }

 /**
  * Sets whether the right side of the control is drawn rounded.
  */
 public void setRoundRight(boolean newRoundRight)
 {
  if (roundRight != newRoundRight)
  {
   roundRight = newRoundRight;
   repaint();
  }
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseClicked(MouseEvent evt) {}

 /**
  * Event handler
  */
 public void mousePressed(MouseEvent evt)
 {
  if (isEnabled())
  {
   mouseIsPressed = true;
   if (!pressedColor.equals(enabledColor))
   {
    repaint();
   }
  }
 }

 /**
  * Event handler
  */
 public void mouseReleased(MouseEvent evt)
 {
  mouseIsPressed = false;
  if ((isEnabled()) && (!pressedColor.equals(enabledColor)))
  {
   repaint();
  }
  fireAction();
 }

 /**
  * Event handler (not implemented)
  */
 public void mouseEntered(MouseEvent evt) {}

 /**
  * Event handler (not implemented)
  */
 public void mouseExited(MouseEvent evt) {}

 /**
  * Sends the Action-Event to all listeners.
  */
 protected void fireAction()
 {
  ActionEvent actionEvt = new ActionEvent(this, 0, null);
  Vector vec;
  synchronized(this)
  {
   vec = (Vector)listeners.clone();
  }
  for (int i = 0; i < vec.size(); i++)
  {
   ((ActionListener)vec.elementAt(i)).actionPerformed(actionEvt);
  }
 }

 /**
  * Adds an Action Listener.
  */
 public synchronized void addActionListener(ActionListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes an Action Listener.
  */
 public synchronized void removeActionListener(ActionListener l)
 {
  listeners.removeElement(l);
 }
 
}