
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A filled half circle (design element).
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsHCircle extends Canvas implements Serializable
{
 /**
  * The "bottom line" of the half circle at the top.
  */
 public static final int caTop = 0;
 /**
  * The "bottom line" of the half circle at the right side.
  */
 public static final int caRight = 1;
 /**
  * The "bottom line" of the half circle at the bottom.
  */
 public static final int caBottom = 2;
 /**
  * The "bottom line" of the half circle at the left side.
  */
 public static final int caLeft = 3;
 
 private int hCircleAlign;
 private Image backImage;
 private Vector listeners = new Vector();

 /**
  * Constructs a new instance.
  */
 public LcarsHCircle()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  hCircleAlign = caRight;
  setForeground(Color.yellow);
  setSize(15, 30);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {}

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  Color foreColor = getForeground();
  int width = getSize().width;
  int height = getSize().height;
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.clearRect(0, 0, width, height);
  backBuffer.setColor(foreColor);
  switch (hCircleAlign)
  {
   case caTop :
    backBuffer.fillArc(0, 0 - height, width, height * 2, 180, 180);
    break;
   case caRight :
    backBuffer.fillArc(0, 0, width * 2, height, 90, 180);
    break;
   case caBottom :
    backBuffer.fillArc(0, 0, width, height * 2, 0, 180);
    break;
   case caLeft :
    backBuffer.fillArc(0 - width, 0, width * 2, height, 270, 180);
    break;
  }
  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Returns the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }
 
 /**
  * Returns the alignment of the half circle.
  */
 public int getHCircleAlign()
 {
  return hCircleAlign;
 }

 /**
  * Sets the alignment of the half circle.<BR>
  * Possible values: caTop, caRight, caBottom, caLeft<BR>
  * If the new value is not in the range between 0 and 3 the property is
  * set to caRight.
  */
 public void setHCircleAlign(int newHCircleAlign)
 {
  if (hCircleAlign != newHCircleAlign)
  {
   if ((newHCircleAlign < 0) || (newHCircleAlign > 3))
    hCircleAlign = caRight;
   else
    hCircleAlign = newHCircleAlign;
   repaint();
  }
 }

}
