
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A panel (design element).<BR>
 * Note: This panel can not act as a container.
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsPanel extends Canvas implements Serializable
{
 /**
  * Label is shown horizontally and vertically centered.
  */
 public static final int lpCenter = 0;
 /**
  * Label is shown in the upper left corner of the control.
  */
 public static final int lpTopLeft = 1;
 /**
  * Label is shown in the upper right corner of the control.
  */
 public static final int lpTopRight = 2;
 /**
  * Label is shown in the lower right corner of the control.
  */
 public static final int lpBottomRight = 3;
 /**
  * Label is shown in the lower left corner of the control.
  */
 public static final int lpBottomLeft = 4;

 private Image backImage;
 private String label;
 private Color fontColor;
 private int labelPosition;
 private int labelMarginX;
 private int labelMarginY;
 private int labelPosX;
 private int labelPosY;
 private Vector listeners = new Vector();

 /**
  * Constructs a new instance.
  */
 public LcarsPanel()
 {
  super();
  try
  {
   jbInit();
  }
  catch (Exception e)
  {
   e.printStackTrace();
  }
  label = "";
  labelMarginX = 5;
  labelMarginY = 2;
  labelPosition = lpCenter;
  fontColor = Color.black;
  setForeground(Color.yellow);
  setSize(80, 30);
  setVisible(true);
 }

 /**
  * Initializes the state of this instance.
  */
 private void jbInit() throws Exception {}

 /**
  * Paints the control.
  */
 public void paint(Graphics g)
 {
  Color foreColor = getForeground();
  int width = getSize().width;
  int height = getSize().height;
  backImage = createImage(width, height);
  Graphics backBuffer = backImage.getGraphics();
  backBuffer.clearRect(0, 0, width, height);
  backBuffer.setColor(foreColor);
  backBuffer.fillRect(0, 0, width, height);
  backBuffer.setColor(fontColor);
  FontMetrics fm = backBuffer.getFontMetrics();
  switch (labelPosition)
  {
   case lpCenter :
    labelPosX = (width - fm.stringWidth(label)) /2;
    labelPosY = (height + fm.getAscent() - fm.getDescent()) / 2;
    break;
   case lpTopLeft :
    labelPosX = labelMarginX;
    labelPosY = labelMarginY + fm.getAscent();
    break;
   case lpTopRight :
    labelPosX = width - labelMarginX - fm.stringWidth(label);
    labelPosY = labelMarginY + fm.getAscent();
    break;
   case lpBottomRight :
    labelPosX = width - labelMarginX - fm.stringWidth(label);
    labelPosY = height - labelMarginY - fm.getDescent();
    break;
   case lpBottomLeft :
    labelPosX = labelMarginX;
    labelPosY = height - labelMarginY - fm.getDescent();
    break;
  }
  backBuffer.drawString(label, labelPosX, labelPosY);
  g.drawImage(backImage, 0, 0, width, height, this);
  backBuffer.dispose();
 }

 /**
  * An overwritten and because of this almost flicker-free
  * version of update().
  */
 public void update(Graphics g)
 {
  paint(g);
 }

 /**
  * Gets the preferred size of the component.
  */
 public Dimension getPreferredSize()
 {
  return getSize();
 }

 /**
  * Returns the color used for drawing the label.
  */
 public Color getFontColor()
 {
  return fontColor;
 }

 /**
  * Sets the color used for drawing the label.
  */
 public void setFontColor(Color newFontColor)
 {
  if (!fontColor.equals(newFontColor))
  {
   fontColor = newFontColor;
   repaint();
  }
 }

 /**
  * Returns the label.
  */
 public String getLabel()
 {
  return label;
 }

 /**
  * Sets the label.
  */
 public void setLabel(String newLabel)
 {
  if (!label.equals(newLabel))
  {
   label = newLabel;
   repaint();
  }
 }

 /**
  * Gets the position of the label.
  */
 public int getLabelPosition()
 {
  return labelPosition;
 }

 /**
  * Sets the position of the label.<BR>
  * Possible values: lpCenter, lpTopLeft, lpTopRight, lpBottomRight, lpBottomLeft<BR>
  * If the new value is not in the range between 0 and 4 the property is set to
  * lpCenter.
  */
 public void setLabelPosition(int newLabelPosition)
 {
  if (labelPosition != newLabelPosition)
  {
   if ((newLabelPosition < 0) || (newLabelPosition > 4))
    labelPosition = lpCenter;
   else
    labelPosition = newLabelPosition;
   repaint();
  }
 }

 /**
  * Gets the horizontal margin between the label and the bound of the control.
  */
 public int getLabelMarginX()
 {
  return labelMarginX;
 }

 /**
  * Sets the horizontal margin between the label and the bound of the control
  * if the label position is not lpCenter.
  */
 public void setLabelMarginX(int newLabelMarginX)
 {
  if (labelMarginX != newLabelMarginX)
  {
   labelMarginX = newLabelMarginX;
   repaint();
  }
 }

 /**
  * Gets the vertical margin between the label and the bound of the control.
  */
 public int getLabelMarginY()
 {
  return labelMarginY;
 }

 /**
  * Sets the vertical margin between the label and the bound of the control
  * if the label position is not lpCenter.
  */
 public void setLabelMarginY(int newLabelMarginY)
 {
  if (labelMarginY != newLabelMarginY)
  {
   labelMarginY = newLabelMarginY;
   repaint();
  }
 }

 /**
  * Adds an Action Listener.
  */
 public synchronized void addActionListener(ActionListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes an Action Listener.
  */
 public synchronized void removeActionListener(ActionListener l)
 {
  listeners.removeElement(l);
 }
 
}

