
package lcarsbeans;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A simple (invisible) timer.<BR>
 * This timer is used by LcarsTrackbar, and has only the
 * functionality needed for this purpose.<BR>
 * I wrote it because my browsers didnt find javax.swing.Timer.
 * <P>
 * @author Andrei Scheibner, www.buffalo-as.de, andrei@buffalo-as.de
 * @version 0.9b, January 2002 (March 2001)
 */
public class LcarsTimer extends Component implements Serializable, Runnable
{
 private volatile Thread timerThread;
 private int delay;
 private int initialDelay;
 private int currentDelay;
 private boolean firstTime;
 private boolean running;
 private Vector listeners = new Vector();

 /**
  * Constructs a new instance.
  */
 public LcarsTimer()
 {
  delay = 100;
  initialDelay = 500;
  currentDelay = 500;
  firstTime = true;
  running = false;
 }

 /**
  * Another constructor.
  */
 public LcarsTimer(int newDelay, int newInitialDelay)
 {
  delay = newDelay;
  initialDelay = newInitialDelay;
  currentDelay = initialDelay;
  firstTime = true;
  running = false;
 }

 /**
  * the run() method needed for every thread.
  */
 public void run()
 {
  Thread thisThread = Thread.currentThread();
  while (timerThread == thisThread)
  {
   try
   {
    Thread.sleep(currentDelay);
   }
   catch (InterruptedException e) {}
   if (firstTime)
   {
    firstTime = false;
    currentDelay = delay;
   }
   fireLcarsTimerFire();
  }
 }

 /**
  * Starts the timer.
  */
 public void start()
 {
  if (timerThread == null)
  {
   firstTime = true;
   currentDelay = initialDelay;
   timerThread = new Thread(this);
   running = true;
   timerThread.start();
  }
 }

 /**
  * Stops the timer.
  */
 public void stop()
 {
  if (timerThread != null)
  {
   timerThread = null;
   running = false;
  }
 }

 /**
  * Returns the delay between two timer events.
  */
 public int getDelay()
 {
  return delay;
 }

 /**
  * Sets the delay between two timer events.
  */
 public void setDelay(int newDelay)
 {
  if (delay != newDelay)
   delay = newDelay;
 }

 /**
  * Returns the delay between the start of the timer and the first timer event.
  */
 public int getInitialDelay()
 {
  return initialDelay;
 }

 /**
  * Sets the delay between the start of the timer and the first timer event.
  */
 public void setInitialDelay(int newInitialDelay)
 {
  if (initialDelay != newInitialDelay)
   initialDelay = newInitialDelay;
 }

 /**
  * Returns whether the timer is currently running (and firing timer events).
  */
 public boolean isRunning()
 {
  return running;
 }

 /**
  * Sends the Change-Event to all listeners.
  */
 protected void fireLcarsTimerFire()
 {
  LcarsTimerFiredEvent evt = new LcarsTimerFiredEvent(this);
  Vector vec;
  synchronized(this)
  {
   vec = (Vector)listeners.clone();
  }
  for (int i = 0; i < vec.size(); i++)
  {
   ((LcarsTimerFireListener)vec.elementAt(i)).lcarsTimerFired(evt);
  }
 }

 /**
  * Adds a listener for the LcarsTimerFireEvent.
  */
 public synchronized void addLcarsTimerFireListener(LcarsTimerFireListener l)
 {
  listeners.addElement(l);
 }

 /**
  * Removes a listener for the LcarsTimerFireEvent.
  */
 public synchronized void removeLcarsTimerFireListener(LcarsTimerFireListener l)
 {
  listeners.removeElement(l);
 }
 
}
